# adaptor for TALGenerator
import setpath
from TAL.TALDefs import parseSubstitution
pos = (None, None)

def push_program(gen, expr):
    gen.pushProgram()

def push_all(gen, expr):
    gen.pushSlots()
    gen.pushProgram()


def metal_define_macro(gen, expr):
    gen.emitDefineMacro(expr)

def metal_use_macro(gen, expr):
    gen.emitUseMacro(expr)

def metal_define_slot(gen, expr):
    gen.emitDefineSlot(expr)

def metal_fill_slot(gen, expr):
    gen.emitFillSlot(expr)

_METAL = {
    "define-macro": (push_program, metal_define_macro),
    "use-macro": (push_all, metal_use_macro),
    "define-slot": (push_program, metal_define_slot),
    "fill-slot": (push_program, metal_fill_slot),
}


def tal_define(gen, expr):
    gen.emitDefines(expr, pos)

def tal_condition(gen, expr):
    gen.emitCondition(expr)

def tal_replace(gen, expr):
    key, expr = parseSubstitution(expr, pos)
    cexpr = gen.compileExpression(expr)
    program = gen.popProgram()
    gen.emit("insertStructure", cexpr, {}, program)

def tal_repeat(gen, expr):
    gen.emitRepeat(expr)

def tal_on_error_before(gen, expr):
    gen.pushProgram()
    gen.pushProgram()

def tal_on_error_after(gen, expr):
    block = self.popProgram()
    key, expr = parseSubstitution(expr, position)
    cexpr = self.compileExpression(expr)
    self.emit("insertStructure", cexpr, {}, [])
    handler = self.popProgram()
    self.emit("onError", block, handler)

_TAL = {
    "define": (tal_define, None),
    "condition": (push_program, tal_condition),
    "content": (push_program, tal_replace),
    "replace": (push_program, tal_replace),
    "repeat": (push_program, tal_repeat),
    #"attributes" is meaningless in TERRY
    "on-error": (tal_on_error_before, tal_on_error_after),
}

TALhandlers = {'tal': _TAL, 'metal': _METAL}
