import setpath

import syntax, config, TextTools
from TAL.TALGenerator import TALGenerator
from TERRY_TAL import TALhandlers
from Products.PageTemplates import TALES
from Products.PageTemplates.Expressions import getEngine

class TERRYError(LookupError):
    pass

class debugGenerator(TALGenerator):
    def pushProgram(self):
        if config.DEBUG: print 'pushing at', self.program[-1]
        self.stack.append(self.program)
        self.program = []

    def popProgram(self):
        program = self.program
        self.program = self.stack.pop()
        if config.DEBUG: print 'popping at', self.program[-1]
        return self.optimize(program)

class generator:
    def __init__(self, text, table=None, handlers=None):
        self.engine = getEngine()
        self.gen = debugGenerator(self.engine)
        self.text = text
        if table is None:
            self.table = TextTools.tag(text, syntax.table())[1]
        else:
            self.table = table
        if handlers is None:
            self.handlers = TALhandlers
        else:
            self.handlers = handlers
        self.lang = self.cmd = self.expr = None
        self.stack = []
        map(self.process, self.table)
        self.program, self.macros = self.gen.getCode()
        if not config.DEBUG:
            del self.engine, self.gen, self.table, self.lang, self.cmd, self.expr, self.stack
    
    def push(self):
        self.stack.append((self.lang, self.cmd, self.expr))
    
    def pop(self):
        self.lang, self.cmd, self.expr = self.stack.pop()
    
    def process(self, row):
        fragment = self.text[row[1]:row[2]]
        if type(row[0]) is type(''):
            self.gen.emitRawText(row[0])
        else:
            if config.DEBUG: print 'doing', row[0].name, row[1:], fragment
            getattr(self, 'do_' + row[0].name)(row, fragment)

    def do_data(self, row, fragment):
        self.gen.emitRawText(fragment)
        
    do_text = do_data
        
    def do_TERRY(self, row, fragment):
        if len(row[3]) == 1:
            # escape
            self.gen.emit('TERRYescape', row[3][0][0])
            return
        self.gen.emit('beginScope')
        map(self.process, row[3])
        self.pop()
        if self.lang:
            lang, cmd = self.lang, self.cmd
            h = self.handlers[self.lang][self.cmd]
            self.lang = None
        else:
            if config.DEBUG: print 'TERRY but no lang!' + str(row)
            lang = None
            h = [None]
        if lang:
            if config.DEBUG: print 'calling %s.%s[-1]: %s' % (lang, cmd, h)
        if h[-1]:
            h[-1](self.gen, self.expr)
        self.gen.emit('endScope')
        
    def do_lang(self, row, fragment):
        if not self.handlers.has_key(fragment):
            raise TERRYError, 'Unknown language: ' + fragment
        self.lang = fragment
        
    def do_cmd(self, row, fragment):
        if not self.handlers[self.lang].has_key(fragment):
            raise TERRYError, 'Language %s does not know this command: %s' % (
                                self.lang, fragment)
        self.cmd = fragment
        
    def do_expr(self, row, fragment):
        self.expr = fragment
        self.gen.emit('scopeCommand', self.lang, self.cmd, self.expr)
        h = self.handlers[self.lang][self.cmd]
        if len(h) > 1 and h[0]:
            h[0](self.gen, self.expr)
        self.push()
