import config
import TextTools, string
tt = TextTools
allowed_chars = string.letters + string.digits + '-_'
# markers
class _Marker:
    def __init__(self, name):
        self.name = name
    def __repr__(self):
        return self.name

TERRY = _Marker('TERRY')
lang = _Marker('lang')
cmd = _Marker('cmd')
expr = _Marker('expr')
data = _Marker('data')
text = _Marker('text')

def _possible_end_of_data(text, start, end, TERRYstart, TERRYend):
    # it would be nice to use re, but then we'd have problems if TERRYstart and
    # TERRYend contain special characters
    t = text[start:end]
    #if config.DEBUG:
    #    print 'looking for end of data in "%s"' % t
    T1 = string.find(t, TERRYstart)
    if T1 == -1: T1 = end
    T2 = string.find(t, TERRYend)
    if T2 == -1: T2 = end
    T = min(T1, T2)
    if T < end:
        #if config.DEBUG:
        #    print 'Found at %s ("%s")' % (T, t[T:T+2])
        return T + start
    else:
        #if config.DEBUG:
        #    print 'Not found'
        return start

# tag table for mxTextTools
def table(TERRYstart=config.TERRYstart, TERRYend=config.TERRYend,
           TERRYescape=config.TERRYescape):
    # wrap the function to pass our correct sequences
    def possible_end_of_data(text, start, end, TERRYstart=TERRYstart,
                                TERRYend=TERRYend):
        return _possible_end_of_data(text, start, end, TERRYstart, TERRYend)

    # tag table representing a single TERRY statement
    statement = (
        (None, tt.Word, TERRYstart),
        (TERRYend, tt.Word, TERRYend, 1, tt.MatchOk),
        (TERRYstart, tt.Word, TERRYescape, 1, tt.MatchOk),
        (lang, tt.AllIn, allowed_chars),
        (None, tt.Is, ':'),
        (cmd, tt.AllIn, allowed_chars),
        (None, tt.IsNot, '"'),
        (None, tt.Is, '"'),
        (expr, tt.AllNotIn, r'\"'),
        ('"', tt.Word, r'\"', 1, -1),  # use \", to quote literal quotes
        ('\\', tt.Is, '\\', 1, -2),    # leave alone any other backslashes
        (None, tt.Is, '"'),
        (None, tt.AllIn, string.whitespace, 1),
        (data, tt.Call, possible_end_of_data, 1),
        (None, tt.Word, TERRYend, 1, tt.MatchOk),
        (TERRY, tt.Table, tt.ThisTable, 1, -2),
        (TERRYstart, tt.Word, TERRYstart, tt.MatchFail, -3),
    )
    
    # tag table to parse a string with multiple TERRY statements inside it
    return (
        (TERRY, tt.Table, statement, 1, 1),
        (text, tt.WordStart, TERRYstart, 1, -1),
        (text, tt.AllNotIn, '', tt.MatchFail, tt.MatchOk), # consume the remainder
    )

if config.DEBUG:

    sample1 = r'''
This is some sample text...
%(lang:cmd "some stuff interpolated..."
  foo bar)%
Demonstrating the power of TERRY
and now, %(lang:cmd "some more stuff interpolated..."
  for something completely different!)%
or perhaps not.
'''

    sample2 = r'''
This is some sample text...
%(lang:cmd "some stuff interpolated..."
  foo bar)%
Demonstrating the power of TERRY
and now, %(lang:cmd "some \"more\" stuff interpolated..."
  for something completely different!
  Yes ladies and gentleman... %(monty:python "flying circus"
  we are nesting commands! %()% Thank you very much )%
  And it even works...
)%
or perhaps not.
'''

    print 'Now running sample1...'
    sample1_tag = tt.tag(sample1, table())

    print 'Now running sample2...'
    sample2_tag = tt.tag(sample2, table())

def pretty_print(taglist, s=None, indent=0):
    if type(taglist) is type(()):
        if taglist[0] == 0:
            return 'Failed tag result'
        elif taglist[0] == 1:
            return ('Successful tag result:\n' +
                     pretty_print(taglist[1], s, indent + config.pretty_print_indent))
        else:
            return 'Not a taglist!'
    elif type(taglist) is type([]):
        for e in taglist:
            if type(e) is not type(()) or len(e) != 4:
                return 'Not a taglist!'
    else:
        return 'Not a taglist!'
    # okay, we're pretty sure it's a taglist now, if we didn't already return
    r = ['']
    for e in taglist:
        e_r = []
        e_r.append(str(e[0]))
        if s:
            text = s[e[1]:e[2]]
            if '\n' in text:
                e_r.append('<\n%s\n>' % text)
            else:
                e_r.append('<%s>' % text)
        else:
            e_r.append('text[%s:%s]' % (e[1], e[2]))
        if e[3]:
            e_r.append(pretty_print(e[3], s, indent + config.pretty_print_indent))
        r.append(string.join(e_r, ' '))
    return string.join(r, '\n' + (' ' * indent))
